@@ -2,7 +2,7 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2017 - ROLI Ltd.
+   Copyright (c) 2022 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
@@ -25,18 +25,19 @@
  The block below describes the properties of this module, and is read by
  the Projucer to automatically generate project code that uses it.
  For details about the syntax and how to create or use a module, see the
- JUCE Module Format.txt file.
+ JUCE Module Format.md file.
 
 
  BEGIN_JUCE_MODULE_DECLARATION
 
   ID:                 juce_audio_devices
   vendor:             juce
-  version:            5.4.6
+  version:            7.0.2
   name:               JUCE audio and MIDI I/O device classes
   description:        Classes to play and record from audio and MIDI I/O devices
   website:            http://www.juce.com/juce
   license:            ISC
+  minimumCppStandard: 14
 
   dependencies:       juce_audio_basics, juce_events
   OSXFrameworks:      CoreAudio CoreMIDI AudioToolbox
@@ -63,9 +64,8 @@
 /** Config: JUCE_USE_WINRT_MIDI
     Enables the use of the Windows Runtime API for MIDI, allowing connections
     to Bluetooth Low Energy devices on Windows 10 version 1809 (October 2018
-    Update) and later. If you enable this flag then older, unsupported,
-    versions of Windows will automatically fall back to using the regular
-    Win32 MIDI API.
+    Update) and later. If you enable this flag then older versions of Windows
+    will automatically fall back to using the regular Win32 MIDI API.
 
     You will need version 10.0.14393.0 of the Windows Standalone SDK to compile
     and you may need to add the path to the WinRT headers. The path to the
@@ -89,21 +89,12 @@
 #endif
 
 /** Config: JUCE_WASAPI
-    Enables WASAPI audio devices (Windows Vista and above). See also the
-    JUCE_WASAPI_EXCLUSIVE flag.
+    Enables WASAPI audio devices (Windows Vista and above).
 */
 #ifndef JUCE_WASAPI
  #define JUCE_WASAPI 1
 #endif
 
-/** Config: JUCE_WASAPI_EXCLUSIVE
-    Enables WASAPI audio devices in exclusive mode (Windows Vista and above).
-*/
-#ifndef JUCE_WASAPI_EXCLUSIVE
- #define JUCE_WASAPI_EXCLUSIVE 0
-#endif
-
-
 /** Config: JUCE_DIRECTSOUND
     Enables DirectSound audio (MS Windows only).
 */
@@ -133,30 +124,27 @@
 #endif
 
 /** Config: JUCE_USE_ANDROID_OBOE
-    ***
-    DEVELOPER PREVIEW - Oboe is currently in developer preview and
-    is in active development. This preview allows for early access
-    and evaluation for developers targeting Android platform.
-    ***
-
-    Enables Oboe devices (Android only, API 16 or above). Requires
-    Oboe repository path to be specified in Android exporter.
+    Enables Oboe devices (Android only).
 */
-
 #ifndef JUCE_USE_ANDROID_OBOE
- #define JUCE_USE_ANDROID_OBOE 0
+ #define JUCE_USE_ANDROID_OBOE 1
 #endif
 
-#if JUCE_USE_ANDROID_OBOE && JUCE_ANDROID_API_VERSION < 16
- #undef JUCE_USE_ANDROID_OBOE
- #define JUCE_USE_ANDROID_OBOE 0
+/** Config: JUCE_USE_OBOE_STABILIZED_CALLBACK
+    If JUCE_USE_ANDROID_OBOE is enabled, enabling this will wrap output audio
+    streams in the oboe::StabilizedCallback class. This class attempts to keep
+    the CPU spinning to avoid it being scaled down on certain devices.
+    (Android only).
+*/
+#ifndef JUCE_USE_ANDROID_OBOE_STABILIZED_CALLBACK
+ #define JUCE_USE_ANDROID_OBOE_STABILIZED_CALLBACK 0
 #endif
 
 /** Config: JUCE_USE_ANDROID_OPENSLES
     Enables OpenSLES devices (Android only).
 */
 #ifndef JUCE_USE_ANDROID_OPENSLES
- #if ! JUCE_USE_ANDROID_OBOE && JUCE_ANDROID_API_VERSION >= 9
+ #if ! JUCE_USE_ANDROID_OBOE
   #define JUCE_USE_ANDROID_OPENSLES 1
  #else
   #define JUCE_USE_ANDROID_OPENSLES 0
@@ -174,6 +162,22 @@
 //==============================================================================
 #include "midi_io/juce_MidiDevices.h"
 #include "midi_io/juce_MidiMessageCollector.h"
+
+namespace juce
+{
+    /** Available modes for the WASAPI audio device.
+
+        Pass one of these to the AudioIODeviceType::createAudioIODeviceType_WASAPI()
+        method to create a WASAPI AudioIODeviceType object in this mode.
+    */
+    enum class WASAPIDeviceMode
+    {
+        shared,
+        exclusive,
+        sharedLowLatency
+    };
+}
+
 #include "audio_io/juce_AudioIODevice.h"
 #include "audio_io/juce_AudioIODeviceType.h"
 #include "audio_io/juce_SystemAudioVolume.h"
