@@ -2,17 +2,16 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2017 - ROLI Ltd.
+   Copyright (c) 2022 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
-   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
-   27th April 2017).
+   By using JUCE, you agree to the terms of both the JUCE 7 End-User License
+   Agreement and JUCE Privacy Policy.
 
-   End User License Agreement: www.juce.com/juce-5-licence
-   Privacy Policy: www.juce.com/juce-5-privacy-policy
+   End User License Agreement: www.juce.com/juce-7-licence
+   Privacy Policy: www.juce.com/juce-privacy-policy
 
    Or: You may also use this code under the terms of the GPL v3 (see
    www.gnu.org/licenses).
@@ -27,40 +26,32 @@
 namespace juce
 {
 
-// Win32NativeFileChooser needs to be a reference counted object as there
-// is no way for the parent to know when the dialog HWND has actually been
-// created without pumping the message thread (which is forbidden when modal
-// loops are disabled). However, the HWND pointer is the only way to cancel
-// the dialog box. This means that the actual native FileChooser HWND may
-// not have been created yet when the user deletes JUCE's FileChooser class. If this
-// occurs the Win32NativeFileChooser will still have a reference count of 1 and will
-// simply delete itself immediately once the HWND will have been created a while later.
-class Win32NativeFileChooser  : public ReferenceCountedObject,
-                                private Thread
+// Implemented in juce_win32_Messaging.cpp
+bool dispatchNextMessageOnSystemQueue (bool returnIfNoPendingMessages);
+
+class Win32NativeFileChooser  : private Thread
 {
 public:
-    using Ptr = ReferenceCountedObjectPtr<Win32NativeFileChooser>;
-
     enum { charsAvailableForResult = 32768 };
 
     Win32NativeFileChooser (Component* parent, int flags, FilePreviewComponent* previewComp,
                             const File& startingFile, const String& titleToUse,
                             const String& filtersToUse)
         : Thread ("Native Win32 FileChooser"),
-          owner (parent), title (titleToUse), filtersString (filtersToUse),
+          owner (parent),
+          title (titleToUse),
+          filtersString (filtersToUse.replaceCharacter (',', ';')),
           selectsDirectories ((flags & FileBrowserComponent::canSelectDirectories)   != 0),
-          selectsFiles       ((flags & FileBrowserComponent::canSelectFiles)         != 0),
           isSave             ((flags & FileBrowserComponent::saveMode)               != 0),
           warnAboutOverwrite ((flags & FileBrowserComponent::warnAboutOverwriting)   != 0),
-          selectMultiple     ((flags & FileBrowserComponent::canSelectMultipleItems) != 0),
-          nativeDialogRef (nullptr), shouldCancel (0)
+          selectMultiple     ((flags & FileBrowserComponent::canSelectMultipleItems) != 0)
     {
         auto parentDirectory = startingFile.getParentDirectory();
 
         // Handle nonexistent root directories in the same way as existing ones
         files.calloc (static_cast<size_t> (charsAvailableForResult) + 1);
 
-        if (startingFile.isDirectory() ||startingFile.isRoot())
+        if (startingFile.isDirectory() || startingFile.isRoot())
         {
             initialPath = startingFile.getFullPathName();
         }
@@ -80,10 +71,15 @@ public:
         }
     }
 
-    ~Win32NativeFileChooser()
+    ~Win32NativeFileChooser() override
     {
         signalThreadShouldExit();
-        waitForThreadToExit (-1);
+
+        while (isThreadRunning())
+        {
+            if (! dispatchNextMessageOnSystemQueue (true))
+                Thread::sleep (1);
+        }
     }
 
     void open (bool async)
@@ -97,10 +93,7 @@ public:
         {
             jassert (! isThreadRunning());
 
-            threadHasReference.reset();
             startThread();
-
-            threadHasReference.wait (-1);
         }
         else
         {
@@ -114,10 +107,10 @@ public:
         ScopedLock lock (deletingDialog);
 
         customComponent = nullptr;
-        shouldCancel.set (1);
+        shouldCancel = true;
 
         if (auto hwnd = nativeDialogRef.get())
-            EndDialog (hwnd, 0);
+            PostMessage (hwnd, WM_CLOSE, 0, 0);
     }
 
     Component* getCustomComponent()    { return customComponent.get(); }
@@ -153,30 +146,251 @@ private:
     };
 
     //==============================================================================
-    Component::SafePointer<Component> owner;
+    const Component::SafePointer<Component> owner;
     String title, filtersString;
     std::unique_ptr<CustomComponentHolder> customComponent;
-    String initialPath, returnedString, defaultExtension;
+    String initialPath, returnedString;
 
-    WaitableEvent threadHasReference;
     CriticalSection deletingDialog;
 
-    bool selectsDirectories, selectsFiles, isSave, warnAboutOverwrite, selectMultiple;
+    bool selectsDirectories, isSave, warnAboutOverwrite, selectMultiple;
 
     HeapBlock<WCHAR> files;
     HeapBlock<WCHAR> filters;
 
-    Atomic<HWND> nativeDialogRef;
-    Atomic<int>  shouldCancel;
+    Atomic<HWND> nativeDialogRef { nullptr };
+    bool shouldCancel = false;
+
+    struct FreeLPWSTR
+    {
+        void operator() (LPWSTR ptr) const noexcept { CoTaskMemFree (ptr); }
+    };
+
+    bool showDialog (IFileDialog& dialog, bool async)
+    {
+        FILEOPENDIALOGOPTIONS flags = {};
+
+        if (FAILED (dialog.GetOptions (&flags)))
+            return false;
+
+        const auto setBit = [] (FILEOPENDIALOGOPTIONS& field, bool value, FILEOPENDIALOGOPTIONS option)
+        {
+            if (value)
+                field |= option;
+            else
+                field &= ~option;
+        };
+
+        setBit (flags, selectsDirectories,         FOS_PICKFOLDERS);
+        setBit (flags, warnAboutOverwrite,         FOS_OVERWRITEPROMPT);
+        setBit (flags, selectMultiple,             FOS_ALLOWMULTISELECT);
+        setBit (flags, customComponent != nullptr, FOS_FORCEPREVIEWPANEON);
+
+        if (FAILED (dialog.SetOptions (flags)) || FAILED (dialog.SetTitle (title.toUTF16())))
+            return false;
+
+        PIDLIST_ABSOLUTE pidl = {};
+
+        if (FAILED (SHParseDisplayName (initialPath.toWideCharPointer(), nullptr, &pidl, SFGAO_FOLDER, nullptr)))
+        {
+            LPWSTR ptr = nullptr;
+            auto result = SHGetKnownFolderPath (FOLDERID_Desktop, 0, nullptr, &ptr);
+            std::unique_ptr<WCHAR, FreeLPWSTR> desktopPath (ptr);
+
+            if (FAILED (result))
+                return false;
+
+            if (FAILED (SHParseDisplayName (desktopPath.get(), nullptr, &pidl, SFGAO_FOLDER, nullptr)))
+                return false;
+        }
+
+        const auto item = [&]
+        {
+            ComSmartPtr<IShellItem> ptr;
+            SHCreateShellItem (nullptr, nullptr, pidl, ptr.resetAndGetPointerAddress());
+            return ptr;
+        }();
+
+        if (item != nullptr)
+        {
+            dialog.SetDefaultFolder (item);
+
+            if (! initialPath.isEmpty())
+                dialog.SetFolder (item);
+        }
+
+        String filename (files.getData());
+
+        if (FAILED (dialog.SetFileName (filename.toWideCharPointer())))
+            return false;
+
+        auto extension = getDefaultFileExtension (filename);
+
+        if (extension.isNotEmpty() && FAILED (dialog.SetDefaultExtension (extension.toWideCharPointer())))
+            return false;
+
+        const COMDLG_FILTERSPEC spec[] { { filtersString.toWideCharPointer(), filtersString.toWideCharPointer() } };
+
+        if (! selectsDirectories && FAILED (dialog.SetFileTypes (numElementsInArray (spec), spec)))
+            return false;
+
+        struct Events  : public ComBaseClassHelper<IFileDialogEvents>
+        {
+            explicit Events (Win32NativeFileChooser& o) : owner (o) {}
+
+            JUCE_COMRESULT OnTypeChange (IFileDialog* d) override                                                 { return updateHwnd (d); }
+            JUCE_COMRESULT OnFolderChanging (IFileDialog* d, IShellItem*) override                                { return updateHwnd (d); }
+            JUCE_COMRESULT OnFileOk (IFileDialog* d) override                                                     { return updateHwnd (d); }
+            JUCE_COMRESULT OnFolderChange (IFileDialog* d) override                                               { return updateHwnd (d); }
+            JUCE_COMRESULT OnSelectionChange (IFileDialog* d) override                                            { return updateHwnd (d); }
+            JUCE_COMRESULT OnShareViolation (IFileDialog* d, IShellItem*, FDE_SHAREVIOLATION_RESPONSE*) override  { return updateHwnd (d); }
+            JUCE_COMRESULT OnOverwrite (IFileDialog* d, IShellItem*, FDE_OVERWRITE_RESPONSE*) override            { return updateHwnd (d); }
+
+            JUCE_COMRESULT updateHwnd (IFileDialog* d)
+            {
+                HWND hwnd = nullptr;
+
+                if (auto window = ComSmartPtr<IFileDialog> { d }.getInterface<IOleWindow>())
+                    window->GetWindow (&hwnd);
+
+                ScopedLock lock (owner.deletingDialog);
+
+                if (owner.shouldCancel)
+                    d->Close (S_FALSE);
+                else if (hwnd != nullptr)
+                    owner.nativeDialogRef = hwnd;
+
+                return S_OK;
+            }
+
+            Win32NativeFileChooser& owner;
+        };
+
+        {
+            ScopedLock lock (deletingDialog);
+
+            if (shouldCancel)
+                return false;
+        }
+
+        const auto result = [&]
+        {
+            struct ScopedAdvise
+            {
+                ScopedAdvise (IFileDialog& d, Events& events) : dialog (d) { dialog.Advise (&events, &cookie); }
+                ~ScopedAdvise() { dialog.Unadvise (cookie); }
+                IFileDialog& dialog;
+                DWORD cookie = 0;
+            };
+
+            Events events { *this };
+            ScopedAdvise scope { dialog, events };
+            return dialog.Show (async ? nullptr : static_cast<HWND> (owner->getWindowHandle())) == S_OK;
+        }();
+
+        ScopedLock lock (deletingDialog);
+        nativeDialogRef = nullptr;
+
+        return result;
+    }
 
     //==============================================================================
-    Array<URL> openDialog (bool async)
+    Array<URL> openDialogVistaAndUp (bool async)
+    {
+        const auto getUrl = [] (IShellItem& item)
+        {
+            LPWSTR ptr = nullptr;
+
+            if (item.GetDisplayName (SIGDN_FILESYSPATH, &ptr) != S_OK)
+                return URL();
+
+            const auto path = std::unique_ptr<WCHAR, FreeLPWSTR> { ptr };
+            return URL (File (String (path.get())));
+        };
+
+        if (isSave)
+        {
+            const auto dialog = [&]
+            {
+                ComSmartPtr<IFileDialog> ptr;
+                ptr.CoCreateInstance (CLSID_FileSaveDialog, CLSCTX_INPROC_SERVER);
+                return ptr;
+            }();
+
+            if (dialog == nullptr)
+                return {};
+
+            showDialog (*dialog, async);
+
+            const auto item = [&]
+            {
+                ComSmartPtr<IShellItem> ptr;
+                dialog->GetResult (ptr.resetAndGetPointerAddress());
+                return ptr;
+            }();
+
+            if (item == nullptr)
+                return {};
+
+            const auto url = getUrl (*item);
+
+            if (url.isEmpty())
+                return {};
+
+            return { url };
+        }
+
+        const auto dialog = [&]
+        {
+            ComSmartPtr<IFileOpenDialog> ptr;
+            ptr.CoCreateInstance (CLSID_FileOpenDialog, CLSCTX_INPROC_SERVER);
+            return ptr;
+        }();
+
+        if (dialog == nullptr)
+            return {};
+
+        showDialog (*dialog, async);
+
+        const auto items = [&]
+        {
+            ComSmartPtr<IShellItemArray> ptr;
+            dialog->GetResults (ptr.resetAndGetPointerAddress());
+            return ptr;
+        }();
+
+        if (items == nullptr)
+            return {};
+
+        Array<URL> result;
+
+        DWORD numItems = 0;
+        items->GetCount (&numItems);
+
+        for (DWORD i = 0; i < numItems; ++i)
+        {
+            ComSmartPtr<IShellItem> scope;
+            items->GetItemAt (i, scope.resetAndGetPointerAddress());
+
+            if (scope != nullptr)
+            {
+                const auto url = getUrl (*scope);
+
+                if (! url.isEmpty())
+                    result.add (url);
+            }
+        }
+
+        return result;
+    }
+
+    Array<URL> openDialogPreVista (bool async)
     {
         Array<URL> selections;
 
         if (selectsDirectories)
         {
-            BROWSEINFO bi = { 0 };
+            BROWSEINFO bi = {};
             bi.hwndOwner = (HWND) (async ? nullptr : owner->getWindowHandle());
             bi.pszDisplayName = files;
             bi.lpszTitle = title.toWideCharPointer();
@@ -213,13 +427,20 @@ private:
         }
         else
         {
-            OPENFILENAMEW of = { 0 };
+            OPENFILENAMEW of = {};
 
-            #ifdef OPENFILENAME_SIZE_VERSION_400W
+           #ifdef OPENFILENAME_SIZE_VERSION_400W
             of.lStructSize = OPENFILENAME_SIZE_VERSION_400W;
            #else
             of.lStructSize = sizeof (of);
            #endif
+
+            if (files[0] != 0)
+            {
+                auto startingFile = File (initialPath).getChildFile (String (files.get()));
+                startingFile.getFullPathName().copyToUTF16 (files, charsAvailableForResult * sizeof (WCHAR));
+            }
+
             of.hwndOwner = (HWND) (async ? nullptr : owner->getWindowHandle());
             of.lpstrFilter = filters.getData();
             of.nFilterIndex = 1;
@@ -233,16 +454,10 @@ private:
 
             if (isSave)
             {
-                StringArray tokens;
-                tokens.addTokens (filtersString, ";,", "\"'");
-                tokens.trim();
-                tokens.removeEmptyStrings();
+                auto extension = getDefaultFileExtension (files.getData());
 
-                if (tokens.size() == 1 && tokens[0].removeCharacters ("*.").isNotEmpty())
-                {
-                    defaultExtension = tokens[0].fromFirstOccurrenceOf (".", false, false);
-                    of.lpstrDefExt = defaultExtension.toWideCharPointer();
-                }
+                if (extension.isNotEmpty())
+                    of.lpstrDefExt = extension.toWideCharPointer();
 
                 if (! GetSaveFileName (&of))
                     return {};
@@ -253,7 +468,7 @@ private:
                     return {};
             }
 
-            if (selectMultiple && of.nFileOffset > 0 && files [of.nFileOffset - 1] == 0)
+            if (selectMultiple && of.nFileOffset > 0 && files[of.nFileOffset - 1] == 0)
             {
                 const WCHAR* filename = files + of.nFileOffset;
 
@@ -269,24 +484,53 @@ private:
             }
         }
 
-        getNativeDialogList().removeValue (this);
-
         return selections;
     }
 
+    Array<URL> openDialog (bool async)
+    {
+        struct Remover
+        {
+            explicit Remover (Win32NativeFileChooser& chooser) : item (chooser) {}
+            ~Remover() { getNativeDialogList().removeValue (&item); }
+
+            Win32NativeFileChooser& item;
+        };
+
+        const Remover remover (*this);
+
+        if (SystemStats::getOperatingSystemType() >= SystemStats::WinVista
+            && customComponent == nullptr)
+        {
+            return openDialogVistaAndUp (async);
+        }
+
+        return openDialogPreVista (async);
+    }
+
     void run() override
     {
-        // as long as the thread is running, don't delete this class
-        Ptr safeThis (this);
-        threadHasReference.signal();
-
-        Array<URL> r = openDialog (true);
-        MessageManager::callAsync ([safeThis, r]
+        results = [&]
         {
-            safeThis->results = r;
+            struct ScopedCoInitialize
+            {
+                // IUnknown_GetWindow will only succeed when instantiated in a single-thread apartment
+                ScopedCoInitialize() { ignoreUnused (CoInitializeEx (nullptr, COINIT_APARTMENTTHREADED | COINIT_DISABLE_OLE1DDE)); }
+                ~ScopedCoInitialize() { CoUninitialize(); }
+            };
 
-            if (safeThis->owner != nullptr)
-                safeThis->owner->exitModalState (r.size() > 0 ? 1 : 0);
+            ScopedCoInitialize scope;
+
+            return openDialog (true);
+        }();
+
+        auto safeOwner = owner;
+        auto resultCode = results.size() > 0 ? 1 : 0;
+
+        MessageManager::callAsync ([resultCode, safeOwner]
+        {
+            if (safeOwner != nullptr)
+                safeOwner->exitModalState (resultCode);
         });
     }
 
@@ -296,9 +540,9 @@ private:
         return dialogs;
     }
 
-    static Win32NativeFileChooser* getNativePointerForDialog (HWND hWnd)
+    static Win32NativeFileChooser* getNativePointerForDialog (HWND hwnd)
     {
-        return getNativeDialogList()[hWnd];
+        return getNativeDialogList()[hwnd];
     }
 
     //==============================================================================
@@ -332,6 +576,23 @@ private:
         return ofFlags;
     }
 
+    String getDefaultFileExtension (const String& filename) const
+    {
+        auto extension = filename.fromLastOccurrenceOf (".", false, false);
+
+        if (extension.isEmpty())
+        {
+            auto tokens = StringArray::fromTokens (filtersString, ";,", "\"'");
+            tokens.trim();
+            tokens.removeEmptyStrings();
+
+            if (tokens.size() == 1 && tokens[0].removeCharacters ("*.").isNotEmpty())
+                extension = tokens[0].fromFirstOccurrenceOf (".", false, false);
+        }
+
+        return extension;
+    }
+
     //==============================================================================
     void initialised (HWND hWnd)
     {
@@ -349,7 +610,7 @@ private:
         ScopedLock lock (deletingDialog);
         getNativeDialogList().set (hdlg, this);
 
-        if (shouldCancel.get() != 0)
+        if (shouldCancel)
         {
             EndDialog (hdlg, 0);
         }
@@ -368,10 +629,10 @@ private:
                 auto screenRectangle = Rectangle<int>::leftTopRightBottom (dialogScreenRect.left,  dialogScreenRect.top,
                                                                            dialogScreenRect.right, dialogScreenRect.bottom);
 
-                auto scale = Desktop::getInstance().getDisplays().findDisplayForRect (screenRectangle, true).scale;
+                auto scale = Desktop::getInstance().getDisplays().getDisplayForRect (screenRectangle, true)->scale;
                 auto physicalComponentWidth = roundToInt (safeCustomComponent->getWidth() * scale);
 
-                SetWindowPos (hdlg, 0, screenRectangle.getX(), screenRectangle.getY(),
+                SetWindowPos (hdlg, nullptr, screenRectangle.getX(), screenRectangle.getY(),
                               physicalComponentWidth + jmax (150, screenRectangle.getWidth()),
                               jmax (150, screenRectangle.getHeight()),
                               SWP_NOACTIVATE | SWP_NOOWNERZORDER | SWP_NOZORDER);
@@ -414,9 +675,9 @@ private:
     {
         ScopedLock lock (deletingDialog);
 
-        if (customComponent != nullptr && shouldCancel.get() == 0)
+        if (customComponent != nullptr && ! shouldCancel)
         {
-            if (FilePreviewComponent* comp = dynamic_cast<FilePreviewComponent*> (customComponent->getChildComponent(0)))
+            if (FilePreviewComponent* comp = dynamic_cast<FilePreviewComponent*> (customComponent->getChildComponent (0)))
             {
                 WCHAR path [MAX_PATH * 2] = { 0 };
                 CommDlg_OpenSave_GetFilePath (hdlg, (LPARAM) &path, MAX_PATH);
@@ -427,12 +688,11 @@ private:
                 }
                 else
                 {
-                    Component::SafePointer<FilePreviewComponent> safeComp (comp);
-
-                    File selectedFile (path);
-                    MessageManager::callAsync ([safeComp, selectedFile]() mutable
+                    MessageManager::callAsync ([safeComp = Component::SafePointer<FilePreviewComponent> { comp },
+                                                selectedFile = File { path }]() mutable
                                                {
-                                                    safeComp->selectedFileChanged (selectedFile);
+                                                    if (safeComp != nullptr)
+                                                        safeComp->selectedFileChanged (selectedFile);
                                                });
                 }
             }
@@ -502,7 +762,7 @@ private:
 
         HWND dialogH = GetParent (hwnd);
 
-        if (dialogH == 0)
+        if (dialogH == nullptr)
             dialogH = hwnd;
 
         return dialogH;
@@ -512,17 +772,17 @@ private:
     JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (Win32NativeFileChooser)
 };
 
-class FileChooser::Native     : public Component,
+class FileChooser::Native     : public std::enable_shared_from_this<Native>,
+                                public Component,
                                 public FileChooser::Pimpl
 {
 public:
-
-    Native (FileChooser& fileChooser, int flags, FilePreviewComponent* previewComp)
+    Native (FileChooser& fileChooser, int flagsIn, FilePreviewComponent* previewComp)
         : owner (fileChooser),
-          nativeFileChooser (new Win32NativeFileChooser (this, flags, previewComp, fileChooser.startingFile,
-                                                         fileChooser.title, fileChooser.filters))
+          nativeFileChooser (std::make_unique<Win32NativeFileChooser> (this, flagsIn, previewComp, fileChooser.startingFile,
+                                                                       fileChooser.title, fileChooser.filters))
     {
-        auto mainMon = Desktop::getInstance().getDisplays().getMainDisplay().userArea;
+        auto mainMon = Desktop::getInstance().getDisplays().getPrimaryDisplay()->userArea;
 
         setBounds (mainMon.getX() + mainMon.getWidth() / 4,
                    mainMon.getY() + mainMon.getHeight() / 4,
@@ -533,35 +793,37 @@ public:
         addToDesktop (0);
     }
 
-    ~Native()
+    ~Native() override
     {
         exitModalState (0);
         nativeFileChooser->cancel();
-        nativeFileChooser = nullptr;
     }
 
     void launch() override
     {
-        SafePointer<Native> safeThis (this);
+        std::weak_ptr<Native> safeThis = shared_from_this();
 
-        enterModalState (true, ModalCallbackFunction::create (
-                         [safeThis] (int)
-                         {
-                             if (safeThis != nullptr)
-                                 safeThis->owner.finished (safeThis->nativeFileChooser->results);
-                         }));
+        enterModalState (true, ModalCallbackFunction::create ([safeThis] (int)
+        {
+            if (auto locked = safeThis.lock())
+                locked->owner.finished (locked->nativeFileChooser->results);
+        }));
 
         nativeFileChooser->open (true);
     }
 
     void runModally() override
     {
+       #if JUCE_MODAL_LOOPS_PERMITTED
         enterModalState (true);
         nativeFileChooser->open (false);
         exitModalState (nativeFileChooser->results.size() > 0 ? 1 : 0);
         nativeFileChooser->cancel();
 
         owner.finished (nativeFileChooser->results);
+       #else
+        jassertfalse;
+       #endif
     }
 
     bool canModalEventBeSentToComponent (const Component* targetComponent) override
@@ -577,7 +839,7 @@ public:
 
 private:
     FileChooser& owner;
-    Win32NativeFileChooser::Ptr nativeFileChooser;
+    std::shared_ptr<Win32NativeFileChooser> nativeFileChooser;
 };
 
 //==============================================================================
@@ -590,10 +852,10 @@ bool FileChooser::isPlatformDialogAvailable()
    #endif
 }
 
-FileChooser::Pimpl* FileChooser::showPlatformDialog (FileChooser& owner, int flags,
-                                                     FilePreviewComponent* preview)
+std::shared_ptr<FileChooser::Pimpl> FileChooser::showPlatformDialog (FileChooser& owner, int flags,
+                                                                     FilePreviewComponent* preview)
 {
-    return new FileChooser::Native (owner, flags, preview);
+    return std::make_shared<FileChooser::Native> (owner, flags, preview);
 }
 
 } // namespace juce
